/*
 * Seti@Home client integrated into panel
 * (C) 2000 Richard Kinder
 *
 * Author: Richard Kinder <r_kinder@yahoo.com>
 *
 * Date: 8/1/00
 *
 * modified: 26/3/00 by Craig Orsinger (CJO) <cjorsinger@earthlink.net>
 *            - Changed single callback to three; one each for reading
 *              state file, user file, and display (message[] array info)
 *            - Added prototypes for static functions (and removed them
 *              from "seti_applet.h").
 *            - Fixed bug in long term (years) printouts of CPU time.
 */

#include <gnome.h>
#include <applet-widget.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "seti_applet.h"
#include "seti_control.h"

#include "seti_applet-logo-red.xpm"
#include "seti_applet-logo-green.xpm"

/* boolean ... is seti running? */
static gint seti_is_running = 0;

static gint update_state_file_timeout_id;
static gint update_user_file_timeout_id;
static gint display_timeout_id;
/*If seti@home has finished and requires user intervention, flash the icon*/
static gint attention_timeout_id;
static gint flash_index = 0;
static gint current_count = 0;

static gint previous_status = SETI_STOPPED;

static gint file_version = FILE_SAH;

static PanelOrientType orientation;

static gint spike_thresholds[12] = { 0, 25, 50, 75, 100, 125, 150, 175, 200, 225, 250, 275 };
static gint spike_colours_a[12] = { 0x55CC00FF, 0x55CC00FF, 0x55CC00FF, 0x55CC00FF, 0xFFEE00FF, 0xFFEE00FF, 0xFFEE00FF, 0xFFEE00FF, 0xFF0000FF, 0xFF0000FF, 0xFF0000FF, 0xFF0000FF };
static gint spike_colours_u[12] = { 0x287A00FF, 0x287A00FF, 0x287A00FF, 0x287A00FF, 0x7A7200FF, 0x7A7200FF, 0x7A7200FF, 0x7A7200FF, 0x7C0000FF, 0x7C0000FF, 0x7C0000FF, 0x7C0000FF };

GdkImlibImage* the_red_radar;
GdkImlibImage* the_green_radar;

setiapplet sa =
{
	NULL,
	-1,
	-1,
	-1,
	NULL,
	10,
	120,
	0,
	19,
	NULL,
	FALSE,
	NULL,
	TRUE,
	0.0,
	{
		VALUE_DISPLAY&VALUE_INVALID,
		VALUE_DISPLAY&VALUE_INVALID,
		VALUE_DISPLAY&VALUE_INVALID,
		VALUE_DISPLAY&VALUE_INVALID,
		VALUE_DISPLAY&VALUE_INVALID
	},
	{
		0.0,
		0.0
	},
	{
		0.0,
		0.0,
		0.0,
		0.0,
		{
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
		},
		1.0
	},
	{
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	}
};

/*
 * (CJO) Prototypes of static functions
 */
static void setup_gaus_canvas_item();
static void setup_spike_canvas_item();
static void setup_percent_canvas_item();
static void setup_units_canvas_item();
static void setup_cputime_canvas_item();
static void applet_change_orient ( GtkWidget *, PanelOrientType, gpointer );
static void check_change_pixmap ( void );
static void start_callback ( void );
static void attention_callback ( void );
static void display_callback ( void );
static void state_file_callback ( void );
static void user_file_callback ( void );
static void parse_state_file ( char* );
static void parse_user_file ( char* );
static void update_display ( gint );
static void re_read_state_file ( void );
static void re_read_user_file ( void );
static gchar *format_percentage_string ( void );
static gchar *format_cpu_string ( void );
static gchar *format_spike_string ( void );

void console_log_message ( gchar *message )
{
	gchar *msg;
	gint msg_len = strlen ( message );

	if ( !sa.applet_gui.seti_console_text )
	{
		if ( !sa.seti_console_buffer )
		{
			if ( message[msg_len-1] == '\n' )
			{
				sa.seti_console_buffer = g_strdup_printf ( "%s", message );
			}
			else
			{
				sa.seti_console_buffer = g_strdup_printf ( "%s\n", message );
			}
		}
		else
		{
			if ( message[msg_len-1] == '\n' )
			{
				gchar *tmp = sa.seti_console_buffer;
				sa.seti_console_buffer = g_strconcat ( sa.seti_console_buffer, message, NULL );
				g_free(tmp);
			}
			else
			{
				gchar *tmp = sa.seti_console_buffer;
				sa.seti_console_buffer = g_strconcat ( sa.seti_console_buffer, message, "\n\0", NULL );
				g_free(tmp);
			}
		}
	}
	else
	{
		if ( message[msg_len-1] == '\n' )
		{
			msg = g_strdup_printf ( "%s", message );
		}
		else
		{
			msg = g_strdup_printf ( "%s\n", message );
		}
		gtk_text_insert ( GTK_TEXT ( sa.applet_gui.seti_console_text ), NULL, NULL, NULL, msg, strlen ( msg ) );
		g_free ( msg );
	}
}

void console_clear()
{
	if ( sa.seti_console_buffer )
	{
		g_free ( sa.seti_console_buffer );
	}
	if ( sa.applet_gui.seti_console_text )
	{
		gtk_text_backward_delete ( GTK_TEXT(sa.applet_gui.seti_console_text), gtk_text_get_length ( GTK_TEXT(sa.applet_gui.seti_console_text) ) );
	}
}

static void
help_cb (AppletWidget *widget, gpointer data)
{
	static GnomeHelpMenuEntry help_entry = {NULL, "index.html"};
	help_entry.name = gnome_app_id;
	gnome_help_display ( NULL, &help_entry );
}

static void
console_cb (AppletWidget *widget, gpointer data)
{
	if ( !sa.applet_gui.seti_console )
	{
		sa.applet_gui.seti_console = (GtkWidget *)create_seti_console();
		gtk_widget_show ( sa.applet_gui.seti_console );
	}
	else
	{
		gtk_widget_show ( sa.applet_gui.seti_console );
	}
}

void
console_close_cb()
{
	if ( sa.applet_gui.seti_console )
	{
		gtk_widget_hide ( sa.applet_gui.seti_console );
	}
}

void
on_SetiAppletProperties_help (GnomePropertyBox *gnomepropertybox,
		gint             arg1,
		gpointer         user_data)
{
	help_cb(NULL,NULL);
}

void
on_SetiAppletProperties_apply (GnomePropertyBox *gnomepropertybox,
		gint             arg1,
		gpointer         user_data)
{
	gchar *new_text;
	gint toggle_status;
	/*final apply on close, prevent multiple calls of this method*/
	if ( arg1 == -1 )
	{
		toggle_status = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.percentage_check) );
		sa.value_mask[PERCENT] |= (toggle_status ? VALUE_DISPLAY : VALUE_HIDE);
		sa.value_mask[PERCENT] &= (toggle_status ? ~VALUE_HIDE : ~VALUE_DISPLAY);
		toggle_status = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.gaussian_check) );
		sa.value_mask[GAUSSIAN] |= toggle_status ? VALUE_DISPLAY : VALUE_HIDE;
		sa.value_mask[GAUSSIAN] &= toggle_status ? ~VALUE_HIDE : ~VALUE_DISPLAY;
		toggle_status = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.spike_check) );
		sa.value_mask[SPIKE] |= toggle_status ? VALUE_DISPLAY : VALUE_HIDE;
		sa.value_mask[SPIKE] &= toggle_status ? ~VALUE_HIDE : ~VALUE_DISPLAY;
		toggle_status = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.units_processed_check) );
		sa.value_mask[UNITS] |= toggle_status ? VALUE_DISPLAY : VALUE_HIDE;
		sa.value_mask[UNITS] &= toggle_status ? ~VALUE_HIDE : ~VALUE_DISPLAY;
		toggle_status = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.total_CPU_check) );
		sa.value_mask[CPUTIME] |= toggle_status ? VALUE_DISPLAY : VALUE_HIDE;
		sa.value_mask[CPUTIME] &= toggle_status ? ~VALUE_HIDE : ~VALUE_DISPLAY;

		new_text = gtk_entry_get_text ( GTK_ENTRY (sa.properties_gui.seti_location_text) );
		if ( sa.seti_dir )
		{
			g_free ( sa.seti_dir );
		}
		sa.seti_dir = g_strdup(new_text);
		sa.update_interval = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON ( sa.properties_gui.ui_update_spin ) );
		sa.file_update_interval = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON ( sa.properties_gui.file_update_spin ) );
		sa.launch_on_start = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.launch_on_start) );
		sa.nice_value = (GTK_ADJUSTMENT(sa.properties_gui.nice_adjust))->value;
		if ( sa.extra_params )
		{
			g_free ( sa.extra_params );
		}
		new_text = gtk_entry_get_text ( GTK_ENTRY (sa.properties_gui.extra_parm_text) );
		sa.extra_params = g_strdup(new_text);
		sa.separate_exe_dir = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(sa.properties_gui.use_diff_dir_exe) );
		if ( sa.separate_exe_dir )
		{
			if ( sa.seti_exe_dir )
			{
				g_free ( sa.seti_exe_dir );
			}
			new_text = gtk_entry_get_text ( GTK_ENTRY (sa.properties_gui.seti_alternate_dir) );
			sa.seti_exe_dir = g_strdup(new_text);
		}
		re_read_state_file();
		re_read_user_file();
		start_callback();
        }
}

static void refresh_tool_tip()
{
	gchar *seti_state;
	gchar *time_string;
	time_string = format_cpu_string();
	seti_state = g_strdup_printf ( "%.2f%%, Spike:%.2f, Gaussian Power %.2f, Gaussian Score %.2f, Units:%d, CPU Time: %s", sa.current_unit_info.percent_complete*100, sa.current_unit_info.best_spike, sa.current_unit_info.best_gaussian_power, sa.current_unit_info.best_gaussian_score, (gint)sa.user_info.units_processed, time_string );
	gtk_tooltips_set_tip ( sa.applet_gui.tip_text, GTK_WIDGET ( sa.applet_gui.the_canvas ), seti_state, "Seti state" );
	gtk_tooltips_enable ( sa.applet_gui.tip_text );
	g_free ( time_string );
	g_free ( seti_state );
}

static void
applet_change_orient ( GtkWidget *widget, PanelOrientType or, gpointer data )
{
	GnomeCanvasGroup* group;
	gdouble image_size = RADAR_DX;

	group = gnome_canvas_root ( GNOME_CANVAS ( sa.applet_gui.the_canvas ) );
	sa.panel_size = applet_widget_get_panel_pixel_size(APPLET_WIDGET(sa.applet_gui.applet))/24;
	if ( sa.panel_size < 2.0 )
	{
		image_size = 14.0;
	}
	orientation = or;
	gtk_tooltips_disable ( sa.applet_gui.tip_text );
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		if ( sa.panel_size <= 1.0 )
		{
			gtk_widget_set_usize ( sa.applet_gui.the_canvas, (sa.panel_size / 2.0 ) * 36, 19 );
			if ( ( sa.applet_gui.title_text_item != NULL ) && ( ! GTK_OBJECT_DESTROYED ( GTK_OBJECT ( sa.applet_gui.title_text_item ) ) ) )
			{
				gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.title_text_item ) );
				sa.applet_gui.title_text_item = 0;
			}
			gnome_canvas_item_set ( sa.applet_gui.radar_red_item, "x", 8.0, "y", 8.0, "width", RADAR_DX-1, "height", RADAR_DY-1, NULL );
			gnome_canvas_item_set ( sa.applet_gui.radar_green_item, "x", 8.0, "y", 8.0, "width", RADAR_DX-1, "height", RADAR_DY-1, NULL );
			refresh_tool_tip();
		}
		else
		{
			gtk_widget_set_usize ( sa.applet_gui.the_canvas, (sa.panel_size / 2.0 ) * 36, 85 );
			if ( ( sa.applet_gui.title_text_item != NULL ) && ! GTK_OBJECT_DESTROYED ( GTK_OBJECT ( sa.applet_gui.title_text_item ) ) )
			{
				gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.title_text_item ) );
				sa.applet_gui.title_text_item = 0;
			}
			sa.applet_gui.title_text_item = GNOME_CANVAS_ITEM (
				GNOME_CANVAS_GROUP( gnome_canvas_item_new ( group,
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL ) ) );
			gnome_canvas_item_new ( GNOME_CANVAS_GROUP (
				sa.applet_gui.title_text_item ),
				gnome_canvas_text_get_type(),
				"text", "seti",
				"x", 17.0,
				"y", -17.0,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_CENTER,
				"fill_color_rgba", 0x003300FF,
				NULL );
			gnome_canvas_item_new ( GNOME_CANVAS_GROUP (
				sa.applet_gui.title_text_item ),
				gnome_canvas_text_get_type(),
				"text", "@",
				"x", 17.0,
				"y", -10.0,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_CENTER,
				"fill_color_rgba", 0x003300FF,
				NULL );
			gnome_canvas_item_new ( GNOME_CANVAS_GROUP (
				sa.applet_gui.title_text_item ),
				gnome_canvas_text_get_type(),
				"text", "home",
				"x", 17.0,
				"y", -3.0,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_CENTER,
				"fill_color_rgba", 0x003300FF,
				NULL );
			gnome_canvas_item_set ( sa.applet_gui.radar_red_item, "x", 17.0, "y", 51.0, "width", RADAR_DX, "height", RADAR_DY, NULL );
			gnome_canvas_item_set ( sa.applet_gui.radar_green_item, "x", 17.0, "y", 51.0, "width", RADAR_DX, "height", RADAR_DY, NULL );
		}
	}
	else
	{
		if ( ( sa.applet_gui.title_text_item != NULL ) && ! GTK_OBJECT_DESTROYED ( GTK_OBJECT ( sa.applet_gui.title_text_item ) ) )
		{
			gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.title_text_item ) );
			sa.applet_gui.title_text_item = 0;
		}
		sa.applet_gui.title_text_item = gnome_canvas_item_new ( group,
			gnome_canvas_text_get_type(),
			"text", "seti@home",
			"x", 25.0,
			"y", 4.0,
			"font", SMALL_FONT,
			"anchor", GTK_ANCHOR_CENTER,
			"fill_color_rgba", 0x003300FF,
			NULL );
		gtk_widget_set_usize ( sa.applet_gui.the_canvas, 110, (sa.panel_size / 2.0 ) * 36 );
		gnome_canvas_item_set ( sa.applet_gui.radar_red_item, "y", (gdouble)(RADAR_Y * (sa.panel_size/2)), NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_red_item, "x", RADAR_X, NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_red_item, "width", image_size, NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_red_item, "height", image_size, NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_green_item, "y", (gdouble)(RADAR_Y * (sa.panel_size/2)), NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_green_item, "x", RADAR_X, NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_green_item, "width", image_size, NULL );
		gnome_canvas_item_set ( sa.applet_gui.radar_green_item, "height", image_size, NULL );
	}
	re_read_state_file();
	re_read_user_file();
	update_display ( 1 );
	start_callback();
}


static void
about_cb (AppletWidget *widget, gpointer data)
{
	GtkWidget *about;
	static const gchar *authors[2];
	authors[0] = "Richard Kinder <r_kinder@yahoo.com>";
	authors[1] = "Matt Herbert <mherbert@bit-net.com>";
	authors[2] = NULL;

	about = gnome_about_new (_("Seti@Home Applet"), VERSION,
			"(C) 2000",
			authors,
			_("An applet which shows the status of the seti@home client within the panel."),
			NULL);
	gtk_widget_show (about);
	return;
}

gchar *
re_read_file ( gchar *full_path )
{
	int fd;
	int stat_rv;
	int num_bytes_read;
	gchar* read_buf = g_malloc0 ( FILE_SIZE );
	struct stat *stat_buf = g_malloc0 ( sizeof ( struct stat ) );

	/* Get file stats, check it exists, has size > 0 */
	stat_rv = stat ( full_path, stat_buf );
	if ( stat_rv == -1 )
	{
		g_free ( read_buf );
		read_buf = 0;
	}
	else
	{
		if ( ( stat_buf->st_size ) == 0 )
		{
			g_free ( read_buf );
			read_buf = 0;
		}
		else
		{

			fd = open ( full_path, O_RDONLY );
			if ( fd != -1 )
			{
				num_bytes_read = read ( fd, read_buf, FILE_SIZE );
				close ( fd );
			}
			else
			{
				g_free ( read_buf );
				read_buf = 0;
			}
		}
	}
	g_free ( stat_buf );
	return read_buf;
}

static void
re_read_state_file()
{
	gchar *file_contents;
	gchar *full_path;
	/*Try the new seti state.sah file name*/
	full_path = g_strdup_printf ( "%s/%s", sa.seti_dir, STATE_SAH );
	file_contents = re_read_file ( full_path );
	/*Try the old seti state.txt file name*/
	if ( !file_contents )
	{
		g_free ( full_path );
		full_path = g_strdup_printf ( "%s/%s", sa.seti_dir, STATE_TXT );
		file_contents = re_read_file ( full_path );
		file_version = FILE_TXT;
	}
	parse_state_file ( file_contents );
	g_free ( full_path );
	/*file_contents may be a null pointer - re_read_file returns null if
	 * the open file returned an error. Only free the memory if we have
	 * a clean open file*/
	if ( file_contents )
	{
		g_free ( file_contents );
	}
}

static void
re_read_user_file()
{
	gchar *file_contents;
	gchar *full_path;
	full_path = g_strdup_printf ( "%s/%s", sa.seti_dir, USER_INFO_SAH );
	file_contents = re_read_file ( full_path );
	if ( !file_contents )
	{
		g_free ( full_path );
		full_path = g_strdup_printf ( "%s/%s", sa.seti_dir, USER_INFO_TXT );
		file_contents = re_read_file ( full_path );
	}
	parse_user_file ( file_contents );
	g_free ( full_path );
	/*As for re_read_state_file - dont try to free a null pointer*/
	if ( file_contents )
	{
		g_free ( file_contents );
	}
}

void
change_pixmap()
{
	if ( ( sa.current_image % 2 ) == 1 )
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_red_item, 1 );
	}
	else
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_green_item, 1 );
	}
	sa.current_image ++;
}

static void
attention_timeout(void)
{
	change_pixmap();
	attention_callback();
}

static void attention_timeout_end(void)
{
	int current_status = seti_status ( &sa );
	if ( attention_timeout_id )
	{
		gtk_timeout_remove ( attention_timeout_id );
	}
	attention_timeout_id = 0;
	check_change_pixmap();
	if ( current_status == SETI_RUNNING )
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_red_item, 1 );
	}
	else if ( current_status == SETI_STOPPED )
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_green_item, 1 );
	}
}

static void
attention_callback(void)
{
	if ( attention_timeout_id )
	{
		gtk_timeout_remove ( attention_timeout_id );
	}
	if ( seti_status ( &sa ) == SETI_RUNNING )
	{
		attention_timeout_id = gtk_timeout_add ( ATTENTION_INTERVAL, (GtkFunction)attention_timeout, NULL );
	}
	else
	{
		sa.current_image = 0;
		check_change_pixmap();
	}
}

static void
get_gaus_values ( char* buf )
{
	gdouble d_temp;
	gchar* c_temp;
	gchar* loc;
	gint i;

	for ( i = 0 ; i < 64 ; i++ )
	{
		c_temp = g_strdup_printf ( "bg_pot %d=", i );
		loc = strstr ( buf, c_temp );
		loc += strlen ( c_temp );
		d_temp = g_strtod ( loc, NULL );
		g_free ( c_temp );
		sa.current_unit_info.gaus_values[i] = d_temp;
		if ( d_temp > sa.current_unit_info.max_gaus )
		{
			sa.current_unit_info.max_gaus = d_temp;
		}
	}
	if ( sa.current_unit_info.max_gaus == 0.0 )
	{
		sa.current_unit_info.max_gaus = 1.0;
	}
}

static void
setup_show_percentage ( GtkWidget* the_canvas )
{
	GnomeCanvasGroup* group;
	group = gnome_canvas_root ( GNOME_CANVAS ( the_canvas ) );
	sa.applet_gui.percentage_item = GNOME_CANVAS_ITEM (
			GNOME_CANVAS_GROUP( gnome_canvas_item_new ( group,
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL ) ) );
	{
		gchar* percentage_label;
		gchar* percentage_value;
		gdouble ypos1,ypos2,xpos2;
		gchar* font_size;
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			percentage_label = _("Prog:");
			ypos1 = 14.0;
			font_size = SMALL_FONT;
			ypos2 = ALIGN_ITEM_2_SIDE_Y;
			xpos2 = ALIGN_ITEM_2_SIDE_X;
		}
		else if ( sa.panel_size >= 2.0 )
		{
			percentage_label = _("Progress:");
			ypos1 = 14.0;
			font_size = LARGE_FONT;
			ypos2 = 25.0;
			xpos2 = 0.0;
		}
		else
		{
			percentage_label = _("Prog:");
			ypos1 = 14.0;
			font_size = SMALL_FONT;
			ypos2 = 14.0;
			xpos2 = ALIGN_ITEM_2_TOP_X;
		}
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP (
					sa.applet_gui.percentage_item ),
				gnome_canvas_text_get_type(),
				"text", percentage_label,
				"x", 0.0,
				"y", ypos1,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
		percentage_value = format_percentage_string();
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP (
					sa.applet_gui.percentage_item ),
				gnome_canvas_text_get_type(),
				"text", percentage_value,
				"x", xpos2,
				"y", ypos2,
				"font", font_size,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x003300FF,
				NULL );
		g_free ( percentage_value );
	}
	sa.applet_gui.current_canvas_item = sa.applet_gui.percentage_item;
}

static gchar*
format_percentage_string()
{
	gchar *percentage;
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		percentage = g_strdup_printf ( "%2.1f%c", sa.current_unit_info.percent_complete * 100, '%' );
	}
	else
	{
		percentage = g_strdup_printf ( "%3.2f%c", sa.current_unit_info.percent_complete * 100, '%' );
	}
	return percentage;
}

static void
setup_show_cputime ( GtkWidget* the_canvas )
{
	GnomeCanvasGroup* group;
	group = gnome_canvas_root ( GNOME_CANVAS ( the_canvas ) );
	sa.applet_gui.cpu_time_item = GNOME_CANVAS_ITEM (
			GNOME_CANVAS_GROUP( gnome_canvas_item_new ( group,
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL ) ) );
	{
		gchar *cpu_label;
		gchar *cpu_value;
		gchar *font_size;
		gdouble xpos2;
		gdouble ypos2;
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			xpos2 = ALIGN_ITEM_2_SIDE_X;
			ypos2 = ALIGN_ITEM_2_SIDE_Y;
			font_size = SMALL_FONT;
			cpu_label = _("CPU:");
		}
		else if ( sa.panel_size >= 2.0 )
		{
			xpos2 = 0.0;
			ypos2 = 25.0;
			font_size = LARGE_FONT;
			cpu_label = _("CPU Time:");
		}
		else
		{
			xpos2 = ALIGN_ITEM_2_TOP_X;
			ypos2 = 14.0;
			font_size = SMALL_FONT;
			cpu_label = _("Time:");
		}
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP ( sa.applet_gui.cpu_time_item ),
				gnome_canvas_text_get_type(),
				"text", cpu_label,
				"x", 0.0,
				"y", 14.0,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
		cpu_value = format_cpu_string();
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP ( sa.applet_gui.cpu_time_item ),
				gnome_canvas_text_get_type(),
				"text", cpu_value,
				"x", xpos2,
				"y", ypos2,
				"font", font_size,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x003300FF,
				NULL );
		g_free ( cpu_value );
	}
	sa.applet_gui.current_canvas_item = sa.applet_gui.cpu_time_item;
}

static gchar
*format_cpu_string()
{
	gchar *cpuinfo;
	gdouble cum_time = sa.user_info.cumulative_cpu_time;
	if ( cum_time > 315440000000 )
	{
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			cpuinfo = g_strdup_printf ( ">10ky" );
		}
		else
		{
			cpuinfo = g_strdup_printf ( ">10000yrs" );
		}
	}
	else if ( cum_time > 31557600 )
	{
		cpuinfo = g_strdup_printf ( "%.2fyrs", cum_time / (60*60*24*365.25) );
	}
	else
	{
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			cpuinfo = g_strdup_printf ( "%3.0fd", cum_time / (60*60*24) );
		}
		else
		{
			cpuinfo = g_strdup_printf ( "%3.0f days", cum_time / (60*60*24) );
		}
	}
	return cpuinfo;
}

static void
setup_show_units ( GtkWidget* the_canvas )
{
	GnomeCanvasGroup* group;
	group = gnome_canvas_root ( GNOME_CANVAS ( the_canvas ) );
	sa.applet_gui.num_units_item = GNOME_CANVAS_ITEM (
			GNOME_CANVAS_GROUP( gnome_canvas_item_new ( group,
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL ) ) );
	{
		gchar* units_label = _( "Units:" );
		gchar* units_value = g_strdup_printf ( "%.0f", sa.user_info.units_processed );
		gchar* font_size;
		gdouble xpos2,ypos2;
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			xpos2 = ALIGN_ITEM_2_SIDE_X;
			ypos2 = ALIGN_ITEM_2_SIDE_Y;
			font_size = SMALL_FONT;
		}
		else if ( sa.panel_size >= 2 )
		{
			xpos2 = 0.0;
			ypos2 = 25.0;
			font_size = LARGE_FONT;
		}
		else
		{
			xpos2 = ALIGN_ITEM_2_TOP_X;
			ypos2 = 14.0;
			font_size = SMALL_FONT;
		}
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP (sa.applet_gui.num_units_item),
				gnome_canvas_text_get_type(),
				"text", units_label,
				"x", 0.0,
				"y", 14.0,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP (sa.applet_gui.num_units_item),
				gnome_canvas_text_get_type(),
				"text", units_value,
				"x", xpos2,
				"y", ypos2,
				"font", font_size,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x003300FF,
				NULL );
		g_free ( units_value );
	}
	sa.applet_gui.current_canvas_item = sa.applet_gui.num_units_item;
}

static void
setup_show_spike ( GtkWidget* the_canvas )
{
	GnomeCanvasGroup* group;
	group = gnome_canvas_root ( GNOME_CANVAS ( the_canvas ) );
	sa.applet_gui.spike_item = GNOME_CANVAS_ITEM (
			GNOME_CANVAS_GROUP( gnome_canvas_item_new ( group,
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL ) ) );
	{
		double xpos = 0.0;
		double xpos2 = 0.0;
		int count = 0;
		gint current_colour;
		gchar* spike_value;
		gchar* font_size;
		gdouble y2,y2t;
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			xpos2 = ALIGN_ITEM_2_SIDE_X;
			y2t = ALIGN_ITEM_2_SIDE_Y;
			font_size = SMALL_FONT;
		}
		else if ( sa.panel_size >= 2.0 )
		{
			xpos2 = 24.0;
			y2 = 33.0;
			y2t = 21.0;
			font_size = SMALL_FONT;
		}
		else
		{
			xpos2 = 24.0;
			y2 = 15.0;
			y2t = 12.0;
			font_size = SMALL_FONT;
		}
		spike_value = format_spike_string();
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.spike_item),
				gnome_canvas_text_get_type(),
				"text", _("Spike:"),
				"x", 0.0,
				"y", ALIGN_ITEM_1_SIDE_Y,
				"font", font_size,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );

		}
		else
		{

			for ( ; count < 12 ; count++ )
			{
				gnome_canvas_item_new (
					GNOME_CANVAS_GROUP(sa.applet_gui.spike_item),
					gnome_canvas_rect_get_type(),
					"x1", xpos,
					"y1", 12.0,
					"x2", xpos + 5.0,
					"y2", y2 + 2,
					"fill_color_rgba", 0x999999FF,
					"outline_color_rgba", 0x999999FF,
					NULL );
				xpos += 7.0;
			}
			xpos = 2.0;
			for ( count = 0 ; count < 12 ; count++ )
			{
				if ( sa.current_unit_info.best_spike > spike_thresholds[count] )
				{
					current_colour = spike_colours_a[count];
				}
				else
				{
					current_colour = spike_colours_u[count];
				}
				gnome_canvas_item_new (
						GNOME_CANVAS_GROUP(sa.applet_gui.spike_item),
						gnome_canvas_rect_get_type(),
						"x1", xpos,
						"y1", 10.0,
						"x2", xpos + 5.0,
						"y2", y2,
						"fill_color_rgba", current_colour,
						"outline_color_rgba", current_colour,
						NULL );
				xpos += 7.0;
			}
			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP (sa.applet_gui.spike_item),
				gnome_canvas_text_get_type(),
				"text", spike_value,
				"x", xpos2,
				"y", y2t,
				"font", font_size,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x999999FF,
				NULL );
		}
		gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.spike_item),
				gnome_canvas_text_get_type(),
				"text", spike_value,
				"x", xpos2 + 1.0,
				"y", y2t+1.0,
				"font", font_size,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
		g_free ( spike_value );
	}
	sa.applet_gui.current_canvas_item = sa.applet_gui.spike_item;
}

static gchar*
format_spike_string()
{
	gchar *spike;

	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		spike = g_strdup_printf ( "%3.0f", sa.current_unit_info.best_spike );
	}
	else
	{
		spike = g_strdup_printf ( "%6.2f", sa.current_unit_info.best_spike );
	}
}

static void
setup_show_gaus ( GtkWidget* the_canvas )
{
	GnomeCanvasGroup* group;

	group = gnome_canvas_root ( GNOME_CANVAS ( the_canvas ) );
	sa.applet_gui.gaus_graph_item = GNOME_CANVAS_ITEM (
			GNOME_CANVAS_GROUP( gnome_canvas_item_new ( group,
				gnome_canvas_group_get_type(),
				"x", 0.0,
				"y", 0.0,
				NULL ) ) );
	{
		gint i;
		gint scale_factor;
		GnomeCanvasPoints* points;
		gdouble theAffine[6];
		gdouble y1,y2t;
		gchar* gaus_power_value;
		gchar* gaus_score_value;
		GnomeCanvasItem* tempItem;
		if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
		{
			gaus_power_value = g_strdup_printf ( "P:%2.2f", sa.current_unit_info.best_gaussian_power );
			gaus_score_value = g_strdup_printf ( "S:%2.2f", sa.current_unit_info.best_gaussian_score );
		}
		else if ( sa.panel_size >= 2.0 )
		{
			scale_factor = 20;
			y1 = 35.0;
			y2t = 15.0;
			gaus_power_value = g_strdup_printf ( "P:%2.2f S:%2.2f", sa.current_unit_info.best_gaussian_power, sa.current_unit_info.best_gaussian_score );
		}
		else
		{
			scale_factor = 10;
			y1 = 18.0;
			y2t = 12.0;
			gaus_power_value = g_strdup_printf ( "P:%2.2f S:%2.2f", sa.current_unit_info.best_gaussian_power, sa.current_unit_info.best_gaussian_score );
		}

		if ( ! ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT ) )
		{
			points = gnome_canvas_points_new ( 256 );
			for ( i = 0 ; i < 512 ; i+=8 )
			{
				points->coords[i] =(i/8)*1.25;
				points->coords[i+1] = 0;
				points->coords[i+2] = (i/8)*1.25;
				points->coords[i+3] = (sa.current_unit_info.gaus_values[i/8] * scale_factor)/sa.current_unit_info.max_gaus;
				points->coords[i+4] = ((i/8)+1)*1.25;
				points->coords[i+5] = (sa.current_unit_info.gaus_values[i/8] * scale_factor)/sa.current_unit_info.max_gaus;
				points->coords[i+6] = ((i/8)+1)*1.25;
				points->coords[i+7] = 0;
			}
			tempItem = gnome_canvas_item_new ( 
				GNOME_CANVAS_GROUP ( sa.applet_gui.gaus_graph_item ),
				gnome_canvas_polygon_get_type(),
				"points", points,
				"width_pixels", 1,
				"fill_color_rgba", 0x999999FF,
				"outline_color_rgba", 0x999999FF,
				NULL );
			art_affine_identity ( theAffine );
			art_affine_rotate ( theAffine, 180.0 );
			gnome_canvas_item_affine_relative ( tempItem, theAffine );
			art_affine_identity ( theAffine );
			art_affine_translate ( theAffine, 81.0, y1 );
			gnome_canvas_item_affine_relative ( tempItem, theAffine );
			tempItem = gnome_canvas_item_new (
				GNOME_CANVAS_GROUP ( sa.applet_gui.gaus_graph_item ),
				gnome_canvas_polygon_get_type(),
				"points", points,
				"width_pixels", 1,
				"fill_color", "red",
				"outline_color", "red",
				NULL );
			art_affine_identity ( theAffine );
			art_affine_rotate ( theAffine, 180.0 );
			gnome_canvas_item_affine_relative ( tempItem, theAffine );
			art_affine_identity ( theAffine );
			art_affine_translate ( theAffine, 83.0, y1-2.0 );
			gnome_canvas_item_affine_relative ( tempItem, theAffine );
			gnome_canvas_points_unref(points);
			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.gaus_graph_item),
				gnome_canvas_text_get_type(),
				"text", gaus_power_value,
				"x", 2.0,
				"y", y2t+1.0,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x999999FF,
				NULL );
			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.gaus_graph_item),
				gnome_canvas_text_get_type(),
				"text", gaus_power_value,
				"x", 3.0,
				"y", y2t,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
		}
		else
		{
			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.gaus_graph_item),
				gnome_canvas_text_get_type(),
				"text", _("Gaus:"),
				"x", 0.0,
				"y", ALIGN_ITEM_1_SIDE_Y,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );

			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.gaus_graph_item),
				gnome_canvas_text_get_type(),
				"text", gaus_power_value,
				"x", 0.0,
				"y", ALIGN_ITEM_2_SIDE_Y,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
			gnome_canvas_item_new (
				GNOME_CANVAS_GROUP(sa.applet_gui.gaus_graph_item),
				gnome_canvas_text_get_type(),
				"text", gaus_score_value,
				"x", 0.0,
				"y", ALIGN_ITEM_3_SIDE_Y,
				"font", SMALL_FONT,
				"anchor", GTK_ANCHOR_WEST,
				"fill_color_rgba", 0x000000FF,
				NULL );
			g_free ( gaus_score_value );
		}
		g_free ( gaus_power_value );
	}
	sa.applet_gui.current_canvas_item = sa.applet_gui.gaus_graph_item;
}

static void
parse_state_file ( char* read_buf )
{
	gdouble d_temp;
	gint attention_tmout;
	gchar *loc;

	/*Check one of the strings containing the values, if it is not null, we
	 * have been through here before, else it is the first time through and
	 * we must read in some values
	if ( ( percentage != NULL ) && ( seti_status ( &sa ) == SETI_STOPPED ) )
	{
		return;
	}
	 */

	if ( !read_buf )
	{
		/*Start the flash callback*/
		attention_timeout();
		sa.value_mask[PERCENT] &= ~VALUE_VALID;
		sa.value_mask[PERCENT] |= VALUE_INVALID;
		sa.value_mask[SPIKE] &= ~VALUE_VALID;
		sa.value_mask[SPIKE] |= VALUE_INVALID;
		sa.value_mask[GAUSSIAN] &= ~VALUE_VALID;
		sa.value_mask[GAUSSIAN] |= VALUE_INVALID;
	}
	else
	{
		if ( attention_timeout_id )
		{
			attention_timeout_end();
		}
		loc = strstr ( read_buf, "prog=" );
		if ( loc == NULL )
		{
			sa.value_mask[PERCENT] &= ~VALUE_VALID;
			sa.value_mask[PERCENT] |= VALUE_INVALID;
		}
		else
		{
			sa.value_mask[PERCENT] &= ~VALUE_INVALID;
			sa.value_mask[PERCENT] |= VALUE_VALID;
			loc += 5;
			d_temp = g_strtod ( loc, NULL );
			sa.current_unit_info.percent_complete = d_temp;
		}
		loc = strstr ( loc, "bs_power=" );
		if ( loc == NULL )
		{
			sa.value_mask[SPIKE] &= ~VALUE_VALID;
			sa.value_mask[SPIKE] |= VALUE_INVALID;
		}
		else
		{
			sa.value_mask[SPIKE] &= ~VALUE_INVALID;
			sa.value_mask[SPIKE] |= VALUE_VALID;
			loc += 9;
			sa.current_unit_info.best_spike = g_strtod ( loc, NULL );
		}
		loc = strstr ( loc, "bg_score=" );
		if ( loc == NULL )
		{
			sa.value_mask[GAUSSIAN] &= ~VALUE_VALID;
			sa.value_mask[GAUSSIAN] |= VALUE_INVALID;
		}
		else
		{
			loc += 9;
			sa.current_unit_info.best_gaussian_score = g_strtod ( loc, NULL );
			loc = strstr ( loc, "bg_power=" );
			if ( loc == NULL )
			{
				sa.value_mask[GAUSSIAN] &= ~VALUE_VALID;
				sa.value_mask[GAUSSIAN] |= VALUE_INVALID;
			}
			else
			{
				sa.value_mask[GAUSSIAN] &= ~VALUE_VALID;
				sa.value_mask[GAUSSIAN] |= VALUE_INVALID;
				loc += 9;
				sa.current_unit_info.best_gaussian_power = g_strtod ( loc, NULL );
			}
		}
		get_gaus_values(loc);
	}
}

static void
parse_user_file ( char* read_buf )
{
	gdouble d_temp;
	gchar *loc;

	if ( !read_buf )
	{
		sa.value_mask[UNITS] &= ~VALUE_VALID;
		sa.value_mask[UNITS] |= VALUE_INVALID;
		sa.value_mask[CPUTIME] &= ~VALUE_VALID;
		sa.value_mask[CPUTIME] |= VALUE_INVALID;
	}
	else
	{
		loc = strstr ( read_buf, "nresults=" );
		if ( loc == NULL )
		{
			sa.value_mask[UNITS] &= ~VALUE_VALID;
			sa.value_mask[UNITS] |= VALUE_INVALID;
		}
		else
		{
			sa.value_mask[UNITS] &= ~VALUE_INVALID;
			sa.value_mask[UNITS] |= VALUE_VALID;
			loc += 9;
			d_temp = g_strtod ( loc, NULL );
			sa.user_info.units_processed = d_temp;
		}

		loc = strstr ( loc, "total_cpu=" );
		if ( loc == NULL )
		{
			sa.value_mask[CPUTIME] &= ~VALUE_VALID;
			sa.value_mask[CPUTIME] |= VALUE_INVALID;
		}
		else
		{
			sa.value_mask[CPUTIME] &= ~VALUE_INVALID;
			sa.value_mask[CPUTIME] |= VALUE_VALID;
			loc += 10;
			d_temp = g_strtod ( loc, NULL );
			sa.user_info.cumulative_cpu_time = d_temp;
		}
	}
}

static void
properties_cb (AppletWidget *widget, gpointer data)
{
	GtkWidget* propertiesDialog;

	propertiesDialog = create_SetiAppletProperties();
	gtk_widget_show ( propertiesDialog );
}

static void
update_display ( gint direction )
{
	gint internal_count = 0;
	check_change_pixmap();
	current_count+= direction;

	if ( current_count < 0 )
	{
		current_count = 4;
	}

	if ( current_count >= 5 )
	{
		current_count = current_count%5;
	}
	while ( ( sa.value_mask[current_count%5] & VALUE_HIDE )
		&& ( internal_count < 6 ) )
	{
		internal_count++;
		current_count++;
	}
	if ( internal_count < 6 )
	{
		/*Remove the previous canvas item*/
		switch ( current_count % 5 )
		{
			case GAUSSIAN:
				setup_gaus_canvas_item();
				break;
			case PERCENT:
				setup_percent_canvas_item();
				break;
			case SPIKE:
				setup_spike_canvas_item();
				break;
			case CPUTIME:
				setup_cputime_canvas_item();
				break;
			case UNITS:
				setup_units_canvas_item();
				break;
		}
	}
	else
	{
		/*Show an error image thingo*/
		/*setup_default_canvas_item();*/
	}
}

static void setup_cputime_canvas_item()
{
	if ( sa.applet_gui.current_canvas_item )
	{
		gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.current_canvas_item ) );
	}
	sa.applet_gui.current_canvas_item = 0;
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		if ( sa.panel_size <= 1.0 )
		{
			return;
		}
	}
	setup_show_cputime ( sa.applet_gui.the_canvas );
}

static void setup_units_canvas_item()
{
	if ( sa.applet_gui.current_canvas_item )
	{
		gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.current_canvas_item ) );
	}
	sa.applet_gui.current_canvas_item = 0;
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		if ( sa.panel_size <= 1.0 )
		{
			return;
		}
	}
	setup_show_units ( sa.applet_gui.the_canvas );
}

static void setup_percent_canvas_item()
{
	if ( sa.applet_gui.current_canvas_item )
	{
		gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.current_canvas_item ) );
	}
	sa.applet_gui.current_canvas_item = 0;
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		if ( sa.panel_size <= 1.0 )
		{
			return;
		}
	}
	setup_show_percentage ( sa.applet_gui.the_canvas );
}

static void setup_spike_canvas_item()
{
	if ( sa.applet_gui.current_canvas_item )
	{
		gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.current_canvas_item ) );
	}
	sa.applet_gui.current_canvas_item = 0;
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		if ( sa.panel_size <= 1.0 )
		{
			return;
		}
	}
	setup_show_spike ( sa.applet_gui.the_canvas );
}

static void setup_gaus_canvas_item()
{
	if ( sa.applet_gui.current_canvas_item )
	{
		gtk_object_destroy ( GTK_OBJECT ( sa.applet_gui.current_canvas_item ) );
	}
	sa.applet_gui.current_canvas_item = 0;
	if ( orientation == ORIENT_LEFT || orientation == ORIENT_RIGHT )
	{
		if ( sa.panel_size <= 1.0 )
		{
			return;
		}
	}
	setup_show_gaus ( sa.applet_gui.the_canvas );
}

static void
display_timeout_up(void)
{
	update_display( -1 );
	display_callback();
}

static void
display_timeout(void)
{
	update_display( 1 );
	display_callback();
}

static void
update_state_file_timeout(void)
{
	re_read_state_file();
	refresh_tool_tip();
	state_file_callback();
}

static void
update_user_file_timeout(void)
{
	re_read_user_file();
	user_file_callback();
}

static void
state_file_callback(void)
{
	gint delay ;
	
	if ( sa.file_update_interval < 1 )
	{
		delay = 120 ;	/* (CJO) changed because 2.4 updates every 60 sec.*/
	}
	else
	{
		delay = sa.file_update_interval ;
	}
	delay *= 1000 ;
	if ( update_state_file_timeout_id )
	{
		gtk_timeout_remove ( update_state_file_timeout_id );
	}
	update_state_file_timeout_id = gtk_timeout_add ( delay, (GtkFunction)update_state_file_timeout, NULL );
}

static void
user_file_callback(void)
{
	gint delay ;
	
	/*
	 * (CJO) for now, use the file update interval, just like the
	 *       state file callback does.
	 */
	if ( sa.file_update_interval < 1 )
	{
		delay = 120 ;	/* (CJO) changed because 2.4 updates every 60 sec.*/
	}
	else
	{
		delay = sa.file_update_interval ;
	}
	if ( update_user_file_timeout_id )
	{
		gtk_timeout_remove ( update_user_file_timeout_id );
	}
	/*
	 * (CJO) - changed timeout to every two minutes , because one hour 
	 *         just seems way longer than necessary.
	 */
	update_user_file_timeout_id = gtk_timeout_add ( 1000 * 60 * 2, (GtkFunction)update_user_file_timeout, NULL );
}

static void
display_callback(void)
{
	gint delay ;
	
	if ( sa.update_interval < 1 )
	{
		/* (CJO) for now, try changing message every 5 sec. */
		delay = 5 ;
	}
	else
	{
		delay = sa.update_interval ;
	}
	if ( display_timeout_id )
	{
		gtk_timeout_remove ( display_timeout_id );
	}
	delay *= 1000 ;
	display_timeout_id = gtk_timeout_add ( delay, (GtkFunction)display_timeout, NULL );
}

static void
start_callback(void)
{
	state_file_callback();
	user_file_callback();
	display_callback();
}

static void applet_get_session ( gchar* path )
{
	if ( sa.seti_dir )
	{
		g_free ( sa.seti_dir );
	}
	gnome_config_push_prefix ( path );
	sa.seti_dir = gnome_config_get_string ( "setiapplet/setidir=/usr/local/seti" );
	sa.update_interval = gnome_config_get_int ( "setiapplet/updateinterval=3" );
	sa.file_update_interval = gnome_config_get_int ( "setiapplet/fileupdateinterval=120" );

	sa.current_image = 0;

	sa.value_mask[PERCENT] |= gnome_config_get_int ( "setiapplet/showpercentage=1" ) ? VALUE_DISPLAY : VALUE_HIDE;
	sa.value_mask[SPIKE] |= gnome_config_get_int ( "setiapplet/showspike=1" ) ? VALUE_DISPLAY : VALUE_HIDE;
	sa.value_mask[GAUSSIAN] |= gnome_config_get_int ( "setiapplet/showgaussian=1" ) ? VALUE_DISPLAY : VALUE_HIDE;
	sa.value_mask[UNITS] |= gnome_config_get_int ( "setiapplet/showunits=1" ) ? VALUE_DISPLAY : VALUE_HIDE;
	sa.value_mask[CPUTIME] |= gnome_config_get_int ( "setiapplet/showcpu=1" ) ? VALUE_DISPLAY : VALUE_HIDE;

	sa.nice_value = gnome_config_get_int ( "setiapplet/nicevalue=19" );
	if ( sa.extra_params )
	{
		g_free ( sa.extra_params );
	}
	sa.extra_params = gnome_config_get_string ( "setiapplet/extraparams=");
	sa.separate_exe_dir = gnome_config_get_int ( "setiapplet/separateexedir=0" );
	if ( sa.separate_exe_dir == TRUE )
	{
		sa.seti_exe_dir = gnome_config_get_string ( "setiapplet/setiexedir" );
	}
	sa.launch_on_start = gnome_config_get_int ( "setiapplet/launchonstart=0" );
	gnome_config_pop_prefix();
}

/* sesion save signal handler*/
static gint
applet_save_session(GtkWidget *w,
		    const char *privcfgpath,
		    const char *globcfgpath)
{	
	gnome_config_push_prefix(privcfgpath);
	gnome_config_set_string("setiapplet/setidir", sa.seti_dir);
	gnome_config_set_int("setiapplet/updateinterval", sa.update_interval);
	gnome_config_set_int("setiapplet/fileupdateinterval", sa.file_update_interval);
	gnome_config_set_int("setiapplet/showspike", sa.value_mask[SPIKE]&VALUE_DISPLAY ? 1 : 0 );
	gnome_config_set_int("setiapplet/showgaussian", sa.value_mask[GAUSSIAN]&VALUE_DISPLAY ? 1 : 0 );
	gnome_config_set_int("setiapplet/showunits", sa.value_mask[UNITS]&VALUE_DISPLAY ? 1 : 0 );
	gnome_config_set_int("setiapplet/showcpu", sa.value_mask[CPUTIME]&VALUE_DISPLAY ? 1 : 0 );
	gnome_config_set_int("setiapplet/showpercentage", sa.value_mask[PERCENT]&VALUE_DISPLAY ? 1 : 0 );
	gnome_config_set_int("setiapplet/nicevalue", sa.nice_value);
	gnome_config_set_string("setiapplet/extraparams", sa.extra_params);
	gnome_config_set_int("setiapplet/separateexedir", sa.separate_exe_dir);
	gnome_config_set_int("setiapplet/launchonstart", sa.launch_on_start);
	if ( sa.separate_exe_dir == TRUE )
	{
		gnome_config_set_string("setiapplet/setiexedir", sa.seti_exe_dir);
	}
	gnome_config_pop_prefix();

	gnome_config_sync();
	/* you need to use the drop_all here since we're all writing to
	   one file, without it, things might not work too well */
	gnome_config_drop_all();
		
	/* make sure you return FALSE, otherwise your applet might not
	   work compeltely, there are very few circumstances where you
	   want to return TRUE. This behaves similiar to GTK events, in
	   that if you return FALSE it means that you haven't done
	   everything yourself, meaning you want the panel to save your
	   other state such as the panel you are on, position,
	   parameter, etc ... */
	return FALSE;
}

/*
 * This code is borrowed from VSA.  It will make "toggling" a menu pick
 * between two values easier.
 */
static void register_applet_callbacks()
{

	AppletWidget *a = APPLET_WIDGET(sa.applet_gui.applet);

	if ( !a )
		return;

	/* About Menu pick */
	applet_widget_register_stock_callback(APPLET_WIDGET(sa.applet_gui.applet),
					      "about",
					      GNOME_STOCK_MENU_ABOUT,
					      _("About..."),
					      about_cb,
					      NULL);
	
	/* Properties Menu Pick */
	applet_widget_register_stock_callback(APPLET_WIDGET(sa.applet_gui.applet),
					      "properties",
					      GNOME_STOCK_MENU_PROP,
					      _("Properties..."),
					      properties_cb,
					      NULL);

	/* Start/Stop Menu Toggle */
	applet_widget_register_callback(APPLET_WIDGET(sa.applet_gui.applet),
					      "start_stop_seti",
					      seti_status(&sa) == SETI_RUNNING ?
					      _("Stop seti...") : _("Start seti..."),
					      start_stop_seti_cb,
					      &sa);

	/* Help Menu Pick */
	applet_widget_register_stock_callback(APPLET_WIDGET(sa.applet_gui.applet),
					      "help",
					      GNOME_STOCK_PIXMAP_HELP,
					      _("Help..."),
					      help_cb,
					      NULL);
	/* Console Menu Pick */
	applet_widget_register_callback(APPLET_WIDGET(sa.applet_gui.applet),
					      "console",
					      _("Console..."),
					      console_cb,
					      NULL);
}

static void unregister_applet_callbacks()
{
	AppletWidget *a = APPLET_WIDGET(sa.applet_gui.applet);
	
        if(!a)
		return;

	applet_widget_unregister_callback(a, "properties");
	applet_widget_unregister_callback(a, "about");
	applet_widget_unregister_callback(a, "help");
	applet_widget_unregister_callback(a, "start_stop_seti");
	applet_widget_unregister_callback(a, "console");
}

/**
 * Hack to change popup menu entries
 * This is borrowed from VAS (Visual Sound Analyzer)
 * Modified 28/2/00 RK, change the image from red->green or green->red
 * depending upon the status of the client app.
 */
void update_applet_menu()
{
	if(!sa.applet_gui.applet)
		return;

	unregister_applet_callbacks();
	register_applet_callbacks();
	update_state_file_timeout();
	check_change_pixmap();
}

/**
 * check_change_pixmap, check if the process is running, change the image
 * appropriately.
 */
static void check_change_pixmap()
{
	int current_status = seti_status ( &sa );
	if ( current_status == SETI_RUNNING && previous_status == SETI_STOPPED )
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_red_item, 1 );
	}
	else if ( current_status == SETI_STOPPED && previous_status == SETI_RUNNING )
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_green_item, 1 );
	}
	else if ( attention_timeout_id && current_status == SETI_STOPPED )
	{
		gnome_canvas_item_lower ( sa.applet_gui.radar_green_item, 1 );
	}
	previous_status = current_status;
}

static void
click_event (GnomeCanvasItem* item, GdkEvent* event, gpointer data)
{
	switch ( event->button.button )
	{
		case 4:
			display_timeout_up();
			break;
		case 5:
			display_timeout();
			break;
	}
}

static void
radar_event(GnomeCanvasItem* item, GdkEvent* event, gpointer data)
{
	switch ( event->type )
	{
		case GDK_BUTTON_PRESS:
			if ( event->button.button == 1 )
			{
				start_stop_seti_cb ( APPLET_WIDGET(sa.applet_gui.applet), &sa );
			}
			break;
		default:
			break;
	}
}

static void
create_applet()
{
	GnomeCanvasGroup* group;
	GtkWidget* scrolledwindow1;

	the_green_radar = gdk_imlib_create_image_from_xpm_data ( seti_applet_logo_green_xpm );
	the_red_radar = gdk_imlib_create_image_from_xpm_data ( seti_applet_logo_red_xpm );

	sa.applet_gui.frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(sa.applet_gui.frame), GTK_SHADOW_IN);
	gtk_widget_show(sa.applet_gui.frame);

	sa.applet_gui.tip_text = gtk_tooltips_new();

	gtk_widget_push_visual (gdk_imlib_get_visual ());
	gtk_widget_push_colormap (gdk_imlib_get_colormap ());

	sa.applet_gui.the_canvas = gnome_canvas_new ();
	gtk_widget_set_usize ( sa.applet_gui.the_canvas, 110, 36 );


	gtk_widget_pop_colormap ();
	gtk_widget_pop_visual ();
		      
	group = gnome_canvas_root ( GNOME_CANVAS ( sa.applet_gui.the_canvas ) );
	sa.applet_gui.title_text_item = gnome_canvas_item_new ( group,
			gnome_canvas_text_get_type(),
			"text", "seti@home",
			"x", 25.0,
			"y", 4.0,
			"font", SMALL_FONT,
			"anchor", GTK_ANCHOR_CENTER,
			"fill_color_rgba", 0x003300FF,
			NULL );

	sa.applet_gui.radar_red_item = gnome_canvas_item_new ( group,
				gnome_canvas_image_get_type(),
				"image", the_red_radar,
				"x", RADAR_X,
				"y", RADAR_Y,
				"width", RADAR_DX,
				"height", RADAR_DY,
				NULL );

	gtk_signal_connect ( GTK_OBJECT ( sa.applet_gui.radar_red_item ), "event",
			(GtkSignalFunc)radar_event,
			NULL );

	sa.applet_gui.radar_green_item = gnome_canvas_item_new ( group,
				gnome_canvas_image_get_type(),
				"image", the_green_radar,
				"x", RADAR_X,
				"y", RADAR_Y,
				"width", RADAR_DX,
				"height", RADAR_DY,
				NULL );

	gtk_signal_connect ( GTK_OBJECT ( sa.applet_gui.radar_green_item ), "event",
			(GtkSignalFunc)radar_event,
			NULL );
	gtk_signal_connect ( GTK_OBJECT ( sa.applet_gui.the_canvas ), "button_press_event", (GtkSignalFunc)click_event, NULL );

	if ( seti_status ( &sa ) == SETI_STOPPED )
	{
		/*Lower the green radar dish to allow the red one to show*/
		gnome_canvas_item_lower ( sa.applet_gui.radar_green_item, 1 );
	}
	/*Green radar will show by default, so we need do nothing if seti is
	started*/

	gtk_container_add(GTK_CONTAINER(sa.applet_gui.frame), sa.applet_gui.the_canvas);
	gnome_canvas_set_scroll_region (GNOME_CANVAS (sa.applet_gui.the_canvas), 0, 0, 110, 40);
	gtk_widget_show (sa.applet_gui.the_canvas);
}

int
main(int argc, char **argv)
{
	/* Initialize the i18n stuff */
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	textdomain (PACKAGE);

	/* intialize, this will basically set up the applet, corba and
	   call gnome_init */
	applet_widget_init("seti_applet", VERSION, argc, argv,
				    NULL, 0, NULL);

	/* create a new applet_widget */
	sa.applet_gui.applet = applet_widget_new("seti_applet");
	/* in the rare case that the communication with the panel
	   failed, error out */
	if (!sa.applet_gui.applet)
	{
		g_error("Can't create applet!\n");
	}
	orientation = ORIENT_UP;
	
	applet_get_session(APPLET_WIDGET(sa.applet_gui.applet)->privcfgpath);
	
	if ( sa.launch_on_start == TRUE )
	{
		if ( seti_status ( &sa ) == SETI_STOPPED )
		{
			start_stop_seti_cb ( APPLET_WIDGET(sa.applet_gui.applet), &sa );
		}
	}

	/* create the widget we are going to put on the applet */
	create_applet();

	/* add the widget to the applet-widget, and thereby actually
	   putting it "onto" the panel */
	applet_widget_add (APPLET_WIDGET (sa.applet_gui.applet), sa.applet_gui.frame);
 
	gtk_signal_connect(GTK_OBJECT(sa.applet_gui.applet),"change_orient",
			GTK_SIGNAL_FUNC(applet_change_orient),
			NULL);
	
	/* bind the session save signal */
	gtk_signal_connect(GTK_OBJECT(sa.applet_gui.applet),"save_session",
			   GTK_SIGNAL_FUNC(applet_save_session),
			   NULL);

	/* register the callbacks for the menu */
	register_applet_callbacks();
	
	/* add the widget to the applet-widget, and thereby actually
	   putting it "onto" the panel */
	gtk_widget_show ( sa.applet_gui.applet );

	/*Get some initial values or error message from the state and user
	  files*/
	re_read_state_file();
	refresh_tool_tip();
	re_read_user_file();
	update_display ( 1 );

	/*Set the callback function to update the info displayed*/
	start_callback();

	/* special corba main loop */
	applet_widget_gtk_main ();

	/* NOT REACHED */
	return 0;
}
