/*
 * SetiApplet.h
 *
 * Description: struct and prototype definitions for seti applet
 *
 * Author: Richard Kinder <r_kinder@yahoo.com>
 *
 * Date: 8/1/00
 *
 * modified: 26/3/00 by Craig Orsinger (CJO) <cjorsinger@earthlink.net>
 *            - removed prototypes for static functions
 */

#include "interface.h"
#include "support.h"
#include "config.h"
#include <applet-widget.h>

#ifndef _SETI_H
#define _SETI_H

/* Filenames for state and user info */
#define USER_INFO_SAH "user_info.sah"
#define USER_INFO_TXT "user_info.txt"
#define STATE_SAH "state.sah"
#define STATE_TXT "state.txt"

#define FILE_SAH 0
#define FILE_TXT 1
#define FILE_SIZE 2000

#define SMALL_FONT "6x10"
#define LARGE_FONT "-misc-fixed-bold-r-*-*-*-140-*-*-*-*-*-*"

/* Canvas item alignment constants */
#define RADAR_X 99.0
#define RADAR_Y 18.0

#define RADAR_DX 20.0
#define RADAR_DY 20.0

#define ALIGN_ITEM_1_SIDE_Y 14.0
#define ALIGN_ITEM_2_SIDE_Y 23.0
#define ALIGN_ITEM_3_SIDE_Y 32.0
#define ALIGN_ITEM_2_SIDE_X 0.0
#define ALIGN_ITEM_2_TOP_X 40.0

/* Indices into the user and unit valid array */
#define PERCENT 0
#define SPIKE 1
#define GAUSSIAN 2
#define UNITS 3
#define CPUTIME 4

#define VALUE_VALID                     0x00000001
#define VALUE_INVALID                   0x00000002
#define VALUE_DISPLAY                   0x00000004
#define VALUE_HIDE                      0x00000008

/*Flags for seti client status*/
#define SETI_CLIENT_STARTED_HERE        0x00000001
#define SETI_CLIENT_RUNNING             0x00000002

/*if 65 seconds elapse and no progress (and seti running) then start flashing the logo*/
#define TIMEOUT_ATTENTION 65

/*1300ms between pixmap changes*/
#define ATTENTION_INTERVAL 1300

typedef struct _setiapplet_gui_properties setiapplet_gui_properties;
struct _setiapplet_gui_properties
{
	GtkWidget *seti_location_text;
	GtkWidget *units_processed_check;
	GtkWidget *total_CPU_check;
	GtkWidget *spike_check;
	GtkWidget *gaussian_check;
	GtkWidget *percentage_check;
	GtkWidget *ui_update_spin;
	GtkWidget *file_update_spin;
	GtkWidget *launch_on_start;
	GtkWidget *nice_scale;
	GtkObject *nice_adjust;
	GtkWidget *extra_parm_text;
	GtkWidget *use_diff_dir_exe;
	GtkWidget *seti_alternate_dir;
};

typedef struct _setiapplet_user_info setiapplet_user_info;
struct _setiapplet_user_info
{
	gdouble units_processed;
	gdouble cumulative_cpu_time;
};

typedef struct _setiapplet_current_unit setiapplet_current_unit;
struct _setiapplet_current_unit
{
	gdouble best_spike;
	gdouble best_gaussian_power;
	gdouble best_gaussian_score;
	gdouble percent_complete;
	gdouble gaus_values[64];
	gdouble max_gaus;
};

typedef struct _setiapplet_gui setiapplet_gui;
struct _setiapplet_gui
{
	GtkWidget *applet;
	GtkWidget *frame;
	GtkWidget *align;
	GtkWidget *the_canvas;
	GtkWidget *image_pixmap;
	GnomeCanvasItem *radar_red_item;
	GnomeCanvasItem *radar_green_item;
	GnomeCanvasItem *title_text_item;
	GnomeCanvasItem *gaus_graph_item;
	GnomeCanvasItem *spike_item;
	GnomeCanvasItem *num_units_item;
	GnomeCanvasItem *cpu_time_item;
	GnomeCanvasItem *percentage_item;
	GnomeCanvasItem *current_canvas_item;
	GtkTooltips *tip_text;
	GtkWidget *seti_console;
	GtkWidget *seti_console_text;
};

typedef struct _setiapplet setiapplet;

struct _setiapplet
{
	/*Directory containing seti client*/
	gchar* seti_dir;

	/*fd 4 piped output from the seti@home process started by this applet*/
	gint seti_output;

	guint watch_in;
	guint watch_hup;

	gchar *seti_console_buffer;

	/*update of UI interval (secs)*/
	gint update_interval;

	/*how often the state file is re-read*/
	gint file_update_interval;

	gint current_image;

	/*Nice level to run the process as*/
	gint nice_value;

	/*Extra command line options to pass the client*/
	gchar* extra_params;

	/*whether the exe is in a separate dir from the seti root dir*/
	gint separate_exe_dir;

	gchar* seti_exe_dir;

	/*whether to attempt to launch seti@home on applet start*/
	gint launch_on_start;

	gdouble panel_size;

	/* which values read in are valid and which to display */
	gint value_mask[5];

	//gint flags;

	/* Structures to store the info for the process and the user */
	setiapplet_user_info user_info;
	setiapplet_current_unit current_unit_info;

	/* GUI structures */
	setiapplet_gui_properties properties_gui;
	setiapplet_gui applet_gui;
};

/*Prototypes*/
gchar * re_read_file ( gchar *full_path );
void update_applet_menu( void );
void on_SetiAppletProperties_apply ( GnomePropertyBox*, gint, gpointer );
void on_SetiAppletProperties_help ( GnomePropertyBox*, gint, gpointer );
void console_close_cb ( void );
void console_log_message ( gchar *message );
void console_clear(void);
#endif
